@@ -7,9 +7,14 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
+ * Copyright (C) 2013-2014 jointly by the following organizations:
+ *     1. California Institute of Technology, Pasadena, CA, USA
+ *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
+ *     3. University of Heidelberg, Heidelberg, Germany
+ *
  * Copyright (C) 2009-2013 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
- *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
+ *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *  
  * Copyright (C) 2006-2008 by the California Institute of Technology,
  *     Pasadena, CA, USA 
@@ -26,7 +31,7 @@
  * ------------------------------------------------------------------------ -->
  *
  * @class SBMLNamespaces
- * @sbmlbrief{core} Class to store SBML level, version and namespace
+ * @sbmlbrief{core} Class to store SBML Level, Version and namespace
  * information.
  *
  * @htmlinclude not-sbml-warning.html
@@ -209,12 +214,7 @@ public:
    * @return a string representing the SBML namespace that reflects the
    * SBML Level and Version specified.
    *
-   * @if notclike @note Because this is a @em static method, the
-   * non-C++ language interfaces for libSBML will contain two variants.  One
-   * will be a static method on the class (i.e., SBMLNamespaces), and the
-   * other will be a standalone top-level function with the name
-   * SBMLNamespaces_getSBMLNamespaceURI(). They are functionally
-   * identical. @endif@~
+   * @copydetails doc_note_static_methods
    */
   static std::string getSBMLNamespaceURI(unsigned int level,
                                          unsigned int version);
@@ -226,16 +226,22 @@ public:
    * 
    * @return a list with supported SBML namespaces. 
    *
-   * @if notclike @note Because this is a @em static method, the
-   * non-C++ language interfaces for libSBML will contain two variants.  One
-   * will be a static method on the class (i.e., SBMLNamespaces), and the
-   * other will be a standalone top-level function with the name
-   * SBMLNamespaces_getSupportedNamespaces(). They are functionally
-   * identical. @endif@~
+   * @copydetails doc_note_static_methods
    */
   static const List* getSupportedNamespaces();
 
 
+  /**
+   * Frees the list of supported namespaces as generated by
+   * getSupportedNamespaces().
+   *
+   * @param supportedNS the list to be freed.
+   *
+   * @copydetails doc_note_static_methods
+   */
+  static void freeSBMLNamespaces(List * supportedNS);
+
+
   /**
    * Returns a string representing the SBML XML namespace of this
    * object.
@@ -245,7 +251,7 @@ public:
    */
   virtual std::string getURI() const;
 
-  
+
   /**
    * Get the SBML Level of this SBMLNamespaces object.
    *
@@ -605,12 +611,7 @@ else
    *
    * @return @c true if the "uri" is one of SBML namespaces, @c false otherwise.
    *
-   * @if notclike @note Because this is a @em static method, the
-   * non-C++ language interfaces for libSBML will contain two variants.  One
-   * will be a static method on the class (i.e., SBMLNamespaces), and the
-   * other will be a standalone top-level function with the name
-   * SBMLNamespaces_isSBMLNamespace(). They are functionally
-   * identical. @endif@~
+   * @copydetails doc_note_static_methods
    */
   static bool isSBMLNamespace(const std::string& uri);
 
@@ -665,41 +666,123 @@ LIBSBML_CPP_NAMESPACE_END
 LIBSBML_CPP_NAMESPACE_BEGIN
 BEGIN_C_DECLS
 
-/* ----------------------------------------------------------------------------
- * See the .cpp file for the documentation of the following functions.
- * --------------------------------------------------------------------------*/
-
+/**
+ * Creates a new SBMLNamespaces_t structure corresponding to the given SBML
+ * @p level and @p version.
+ *
+ * SBMLNamespaces_t structures are used in libSBML to communicate SBML Level
+ * and Version data between constructors and other methods.  The
+ * SBMLNamespaces_t structure class tracks 3-tuples (triples) consisting of
+ * SBML Level, Version, and the corresponding SBML XML namespace.  Most
+ * constructors for SBML structures in libSBML take a SBMLNamespaces_t structure
+ * as an argument, thereby allowing the constructor to produce the proper
+ * combination of attributes and other internal data structures for the
+ * given SBML Level and Version.
+ *
+ * The plural name "SBMLNamespaces" is not a mistake, because in SBML
+ * Level&nbsp;3, structures may have extensions added by Level&nbsp;3
+ * packages used by a given model; however, until the introduction of
+ * SBML Level&nbsp;3, the SBMLNamespaces_t structure only records one SBML
+ * Level/Version/namespace combination at a time.
+ *
+ * @param level the SBML level
+ * @param version the SBML version
+ *
+ * @return SBMLNamespaces_t structure created
+ *
+ * @if notcpp @htmlinclude warn-default-args-in-docs.html @endif@~
+ *
+ * @memberof SBMLNamespaces_t
+ */
 LIBSBML_EXTERN
 SBMLNamespaces_t *
 SBMLNamespaces_create(unsigned int level, unsigned int version);
 
 
+/**
+ * Get the SBML Level of this SBMLNamespaces_t structure.
+ *
+ * @param sbmlns the SBMLNamespaces_t structure to query
+ *
+ * @return the SBML Level of this SBMLNamespaces_t structure.
+ *
+ * @memberof SBMLNamespaces_t
+ */
 LIBSBML_EXTERN
 unsigned int
 SBMLNamespaces_getLevel(SBMLNamespaces_t *sbmlns);
 
 
+/**
+ * Get the SBML Version of this SBMLNamespaces_t structure.
+ *
+ * @param sbmlns the SBMLNamespaces_t structure to query
+ *
+ * @return the SBML Version of this SBMLNamespaces_t structure.
+ *
+ * @memberof SBMLNamespaces_t
+ */
 LIBSBML_EXTERN
 unsigned int
 SBMLNamespaces_getVersion(SBMLNamespaces_t *sbmlns);
 
 
+/**
+ * Get the SBML Version of this SBMLNamespaces_t structure.
+ *
+ * @param sbmlns the SBMLNamespaces_t structure to query
+ *
+ * @return the XMLNamespaces_t structure of this SBMLNamespaces_t structure.
+ *
+ * @memberof SBMLNamespaces_t
+ */
 LIBSBML_EXTERN
 XMLNamespaces_t *
 SBMLNamespaces_getNamespaces(SBMLNamespaces_t *sbmlns);
 
 
+/**
+ * Returns a string representing the SBML XML namespace for the 
+ * given @p level and @p version of SBML.
+ *
+ * @param level the SBML level
+ * @param version the SBML version
+ *
+ * @return a string representing the SBML namespace that reflects the
+ * SBML Level and Version specified.
+ *
+ * @memberof SBMLNamespaces_t
+ */
 LIBSBML_EXTERN
 char *
 SBMLNamespaces_getSBMLNamespaceURI(unsigned int level, unsigned int version);
 
 
+/**
+ * Add the XML namespaces list to the set of namespaces
+ * within this SBMLNamespaces_t structure.
+ * 
+ * @param sbmlns the SBMLNamespaces_t structure to add to
+ * @param xmlns the XML namespaces to be added.
+ *
+ * @memberof SBMLNamespaces_t
+ */
 LIBSBML_EXTERN
 int
 SBMLNamespaces_addNamespaces(SBMLNamespaces_t *sbmlns,
                              const XMLNamespaces_t * xmlns);
 
 
+/**
+ * Returns an array of SBML Namespaces supported by this version of 
+ * LibSBML. 
+ *
+ * @param length an integer holding the length of the array
+ * @return an array of SBML namespaces, or @c NULL if length is @c NULL. The array 
+ *         has to be freed by the caller.
+ *
+ * @memberof SBMLNamespaces_t
+ */
 LIBSBML_EXTERN
 SBMLNamespaces_t **
 SBMLNamespaces_getSupportedNamespaces(int *length);
