@@ -9,9 +9,14 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
+ * Copyright (C) 2013-2014 jointly by the following organizations:
+ *     1. California Institute of Technology, Pasadena, CA, USA
+ *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
+ *     3. University of Heidelberg, Heidelberg, Germany
+ *
  * Copyright (C) 2009-2013 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
- *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
+ *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *  
  * Copyright (C) 2006-2008 by the California Institute of Technology,
  *     Pasadena, CA, USA 
@@ -156,9 +161,11 @@ SBasePlugin::getElementBySId(const std::string& id)
     SBase* obj = (SBase*)(allElementsWithIds->get(i));
     if (obj->getId() == id)
     {
+      delete allElementsWithIds;
       return obj;
      }
   }
+  delete allElementsWithIds;
   return NULL;
 }
 
@@ -175,9 +182,12 @@ SBasePlugin::getElementByMetaId(const std::string& metaid)
     SBase* obj = (SBase*)(allElementsWithIds->get(i));
     if (obj->getMetaId() == metaid)
     {
+      delete allElementsWithIds;
       return obj;
-     }
+    }
   }
+
+  delete allElementsWithIds;
   return NULL;
 }
 
@@ -760,17 +770,8 @@ SBasePlugin::logEmptyString(const std::string &attribute,
 
 
 #endif /* __cplusplus */
-/** @cond doxygenCOnly */
+/** @cond doxygenIgnored */
 
-/**
- * Returns the XML namespace (URI) of the package extension
- * of the given plugin structure.
- *
- * @param plugin the plugin structure
- * 
- * @return the URI of the package extension of this plugin object, or NULL
- * in case an invalid plugin structure is provided. 
- */
 LIBSBML_EXTERN
 const char* 
 SBasePlugin_getURI(SBasePlugin_t* plugin)
@@ -779,14 +780,6 @@ SBasePlugin_getURI(SBasePlugin_t* plugin)
   return plugin->getElementNamespace().c_str();
 }
 
-/**
- * Returns the prefix of the given plugin structure.
- *
- * @param plugin the plugin structure
- * 
- * @return the prefix of the given plugin structure, or NULL
- * in case an invalid plugin structure is provided. 
- */
 LIBSBML_EXTERN
 const char* 
 SBasePlugin_getPrefix(SBasePlugin_t* plugin)
@@ -795,14 +788,6 @@ SBasePlugin_getPrefix(SBasePlugin_t* plugin)
   return plugin->getPrefix().c_str();
 }
 
-/**
- * Returns the package name of the given plugin structure.
- *
- * @param plugin the plugin structure
- * 
- * @return the package name of the given plugin structure, or NULL
- * in case an invalid plugin structure is provided. 
- */
 LIBSBML_EXTERN
 const char* 
 SBasePlugin_getPackageName(SBasePlugin_t* plugin)
@@ -811,13 +796,6 @@ SBasePlugin_getPackageName(SBasePlugin_t* plugin)
   return plugin->getPackageName().c_str();
 }
 
-/**
- * Creates a deep copy of the given SBasePlugin_t structure
- * 
- * @param plugin the SBasePlugin_t structure to be copied
- * 
- * @return a (deep) copy of the given SBasePlugin_t structure.
- */
 LIBSBML_EXTERN
 SBasePlugin_t*
 SBasePlugin_clone(SBasePlugin_t* plugin)
@@ -826,18 +804,6 @@ SBasePlugin_clone(SBasePlugin_t* plugin)
   return plugin->clone();
 }
 
-/**
- * Frees the given SBasePlugin_t structure
- * 
- * @param plugin the SBasePlugin_t structure to be freed
- * 
- * @return integer value indicating success/failure of the
- * function.  @if clike The value is drawn from the
- * enumeration #OperationReturnValues_t. @endif@~ The possible values
- * returned by this function are:
- * @li @link OperationReturnValues_t#LIBSBML_OPERATION_SUCCESS LIBSBML_OPERATION_SUCCESS @endlink
- * @li @link OperationReturnValues_t#LIBSBML_INVALID_OBJECT LIBSBML_INVALID_OBJECT @endlink
- */
 LIBSBML_EXTERN
 int
 SBasePlugin_free(SBasePlugin_t* plugin)
@@ -848,18 +814,6 @@ SBasePlugin_free(SBasePlugin_t* plugin)
 }
 
 
-/**
- * Subclasses must override this method to create, store, and then
- * return an SBML object corresponding to the next XMLToken in the
- * XMLInputStream if they have their specific elements.
- *
- * @param plugin the SBasePlugin_t structure 
- * @param stream the XMLInputStream_t structure to read from
- *
- * @return the SBML object corresponding to next XMLToken in the
- * XMLInputStream or NULL if the token was not recognized or plugin or stream 
- * were NULL.
- */
 LIBSBML_EXTERN
 SBase_t*
 SBasePlugin_createObject(SBasePlugin_t* plugin, XMLInputStream_t* stream)
@@ -868,19 +822,6 @@ SBasePlugin_createObject(SBasePlugin_t* plugin, XMLInputStream_t* stream)
   return plugin->createObject(*stream);
 }
 
-/**
- * Subclasses should override this method to read (and store) XHTML,
- * MathML, etc. directly from the XMLInputStream if the target elements
- * can't be parsed by SBase::readAnnotation() and/or SBase::readNotes() 
- * functions
- *
- * @param plugin the SBasePlugin_t structure 
- * @param parentObject the SBase_t structure that will store the annotation.
- * @param stream the XMLInputStream_t structure to read from
- *
- * @return true (1) if the subclass read from the stream, false (0) otherwise. 
- * If an invalid plugin or stream was provided LIBSBML_INVALID_OBJECT is returned.
- */
 LIBSBML_EXTERN
 int
 SBasePlugin_readOtherXML(SBasePlugin_t* plugin, SBase_t* parentObject, XMLInputStream_t* stream)
@@ -889,20 +830,6 @@ SBasePlugin_readOtherXML(SBasePlugin_t* plugin, SBase_t* parentObject, XMLInputS
   return plugin->readOtherXML(parentObject, *stream);
 }
 
-/**
- * Subclasses must override this method to write out their contained
- * SBML objects as XML elements if they have their specific elements.
- *
- * @param plugin the SBasePlugin_t structure  
- * @param stream the XMLOutputStream_t structure to write to
- *
- * @return integer value indicating success/failure of the
- * function.  @if clike The value is drawn from the
- * enumeration #OperationReturnValues_t. @endif@~ The possible values
- * returned by this function are:
- * @li @link OperationReturnValues_t#LIBSBML_OPERATION_SUCCESS LIBSBML_OPERATION_SUCCESS @endlink
- * @li @link OperationReturnValues_t#LIBSBML_INVALID_OBJECT LIBSBML_INVALID_OBJECT @endlink
- */
 LIBSBML_EXTERN
 int
 SBasePlugin_writeElements(SBasePlugin_t* plugin, XMLOutputStream_t* stream)
@@ -912,18 +839,6 @@ SBasePlugin_writeElements(SBasePlugin_t* plugin, XMLOutputStream_t* stream)
   return LIBSBML_OPERATION_SUCCESS;
 }
 
-/** 
- * Checks if the plugin structure has all the required elements.
- *
- * Subclasses should override this function if they have their specific 
- * elements.
- *
- * @param plugin the SBasePlugin_t structure  
- * 
- * @return true (1) if this pugin object has all the required elements,
- * otherwise false (0) will be returned. If an invalid plugin 
- * was provided LIBSBML_INVALID_OBJECT is returned.
- */
 LIBSBML_EXTERN
 int
 SBasePlugin_hasRequiredElements(SBasePlugin_t* plugin)
@@ -932,22 +847,6 @@ SBasePlugin_hasRequiredElements(SBasePlugin_t* plugin)
   return plugin->hasRequiredElements();
 }
 
-/**
- * Subclasses should override this method to get the list of
- * expected attributes if they have their specific attributes.
- * This function is invoked from corresponding readAttributes()
- * function.
- * 
- * @param plugin the SBasePlugin_t structure  
- * @param attributes the ExpectedAttributes_t structure  
- * 
- * @return integer value indicating success/failure of the
- * function.  @if clike The value is drawn from the
- * enumeration #OperationReturnValues_t. @endif@~ The possible values
- * returned by this function are:
- * @li @link OperationReturnValues_t#LIBSBML_OPERATION_SUCCESS LIBSBML_OPERATION_SUCCESS @endlink
- * @li @link OperationReturnValues_t#LIBSBML_INVALID_OBJECT LIBSBML_INVALID_OBJECT @endlink
- */
 LIBSBML_EXTERN
 int
 SBasePlugin_addExpectedAttributes(SBasePlugin_t* plugin, ExpectedAttributes_t* attributes)
@@ -957,21 +856,6 @@ SBasePlugin_addExpectedAttributes(SBasePlugin_t* plugin, ExpectedAttributes_t* a
   return LIBSBML_OPERATION_SUCCESS;
 }
 
-/**
- * Subclasses must override this method to read values from the given
- * XMLAttributes if they have their specific attributes.
- * 
- * @param plugin the SBasePlugin_t structure  
- * @param attributes the XMLAttributes_t structure  
- * @param expectedAttributes the ExpectedAttributes_t structure  
- * 
- * @return integer value indicating success/failure of the
- * function.  @if clike The value is drawn from the
- * enumeration #OperationReturnValues_t. @endif@~ The possible values
- * returned by this function are:
- * @li @link OperationReturnValues_t#LIBSBML_OPERATION_SUCCESS LIBSBML_OPERATION_SUCCESS @endlink
- * @li @link OperationReturnValues_t#LIBSBML_INVALID_OBJECT LIBSBML_INVALID_OBJECT @endlink
- */
 LIBSBML_EXTERN
 int
 SBasePlugin_readAttributes(SBasePlugin_t* plugin, XMLAttributes_t* attributes, 
@@ -983,20 +867,6 @@ SBasePlugin_readAttributes(SBasePlugin_t* plugin, XMLAttributes_t* attributes,
   return LIBSBML_OPERATION_SUCCESS;
 }
 
-/**
- * Subclasses must override this method to write their XML attributes
- * to the XMLOutputStream if they have their specific attributes.
- * 
- * @param plugin the SBasePlugin_t structure  
- * @param stream the XMLOutputStream_t structure  
- * 
- * @return integer value indicating success/failure of the
- * function.  @if clike The value is drawn from the
- * enumeration #OperationReturnValues_t. @endif@~ The possible values
- * returned by this function are:
- * @li @link OperationReturnValues_t#LIBSBML_OPERATION_SUCCESS LIBSBML_OPERATION_SUCCESS @endlink
- * @li @link OperationReturnValues_t#LIBSBML_INVALID_OBJECT LIBSBML_INVALID_OBJECT @endlink
- */
 LIBSBML_EXTERN
 int
 SBasePlugin_writeAttributes(SBasePlugin_t* plugin, XMLOutputStream_t* stream)
@@ -1006,18 +876,6 @@ SBasePlugin_writeAttributes(SBasePlugin_t* plugin, XMLOutputStream_t* stream)
   return LIBSBML_OPERATION_SUCCESS;
 }
 
-/** 
- * Checks if the plugin structure has all the required attributes.
- *
- * Subclasses should override this function if they have their specific 
- * attributes.
- *
- * @param plugin the SBasePlugin_t structure  
- * 
- * @return true (1) if this pugin object has all the required attributes,
- * otherwise false (0) will be returned. If an invalid plugin 
- * was provided LIBSBML_INVALID_OBJECT is returned.
- */
 LIBSBML_EXTERN
 int
 SBasePlugin_hasRequiredAttributes(SBasePlugin_t* plugin)
@@ -1026,24 +884,6 @@ SBasePlugin_hasRequiredAttributes(SBasePlugin_t* plugin)
   return plugin->hasRequiredAttributes();
 }
 
-/**
- * Subclasses should override this method to write required xmlns attributes
- * to the XMLOutputStream (if any). 
- * The xmlns attribute will be written in the element to which the object
- * is connected. For example, xmlns attributes written by this function will
- * be added to Model element if this plugin object connected to the Model 
- * element.
- * 
- * @param plugin the SBasePlugin_t structure  
- * @param stream the XMLOutputStream_t structure  
- * 
- * @return integer value indicating success/failure of the
- * function.  @if clike The value is drawn from the
- * enumeration #OperationReturnValues_t. @endif@~ The possible values
- * returned by this function are:
- * @li @link OperationReturnValues_t#LIBSBML_OPERATION_SUCCESS LIBSBML_OPERATION_SUCCESS @endlink
- * @li @link OperationReturnValues_t#LIBSBML_INVALID_OBJECT LIBSBML_INVALID_OBJECT @endlink
- */
 LIBSBML_EXTERN
 int
 SBasePlugin_writeXMLNS(SBasePlugin_t* plugin, XMLOutputStream_t* stream)
@@ -1053,25 +893,6 @@ SBasePlugin_writeXMLNS(SBasePlugin_t* plugin, XMLOutputStream_t* stream)
   return LIBSBML_OPERATION_SUCCESS;
 }
 
-/**
- * Sets the parent SBMLDocument of th plugin structure.
- *
- * Subclasses which contain one or more SBase derived elements must
- * override this function.
- *
- * @param plugin the SBasePlugin_t structure  
- * @param d the SBMLDocument object to use
- *
- * @see SBasePlugin_connectToParent
- * @see SBasePlugin_enablePackageInternal
- *
- * @return integer value indicating success/failure of the
- * function.  @if clike The value is drawn from the
- * enumeration #OperationReturnValues_t. @endif@~ The possible values
- * returned by this function are:
- * @li @link OperationReturnValues_t#LIBSBML_OPERATION_SUCCESS LIBSBML_OPERATION_SUCCESS @endlink
- * @li @link OperationReturnValues_t#LIBSBML_INVALID_OBJECT LIBSBML_INVALID_OBJECT @endlink
- */
 LIBSBML_EXTERN
 int
 SBasePlugin_setSBMLDocument(SBasePlugin_t* plugin, SBMLDocument_t* d)
@@ -1081,30 +902,6 @@ SBasePlugin_setSBMLDocument(SBasePlugin_t* plugin, SBMLDocument_t* d)
   return LIBSBML_OPERATION_SUCCESS;
 }
 
-/**
- * Sets the parent SBML object of this plugin structure to
- * this object and child elements (if any).
- * (Creates a child-parent relationship by this plugin object)
- *
- * This function is called when this object is created by
- * the parent element.
- * Subclasses must override this this function if they have one
- * or more child elements. Also, SBasePlugin::connectToParent()
- * must be called in the overridden function.
- *
- * @param plugin the SBasePlugin_t structure  
- * @param sbase the SBase_t structure to use
- *
- * @see SBasePlugin_setSBMLDocument
- * @see SBasePlugin_enablePackageInternal
- *
- * @return integer value indicating success/failure of the
- * function.  @if clike The value is drawn from the
- * enumeration #OperationReturnValues_t. @endif@~ The possible values
- * returned by this function are:
- * @li @link OperationReturnValues_t#LIBSBML_OPERATION_SUCCESS LIBSBML_OPERATION_SUCCESS @endlink
- * @li @link OperationReturnValues_t#LIBSBML_INVALID_OBJECT LIBSBML_INVALID_OBJECT @endlink
- */
 LIBSBML_EXTERN
 int
 SBasePlugin_connectToParent(SBasePlugin_t* plugin, SBase_t* sbase)
@@ -1114,31 +911,6 @@ SBasePlugin_connectToParent(SBasePlugin_t* plugin, SBase_t* sbase)
   return LIBSBML_OPERATION_SUCCESS;
 }
 
-/**
- * Enables/Disables the given package with child elements in this plugin 
- * object (if any).
- * (This is an internal implementation invoked from 
- *  SBase::enablePackageInternal() function)
- *
- * Subclasses which contain one or more SBase derived elements should 
- * override this function if elements defined in them can be extended by
- * some other package extension.
- *
- * @param plugin the SBasePlugin_t structure  
- * @param pkgURI the package uri
- * @param pkgPrefix the package prefix
- * @param flag indicating whether the package should be enabled (1) or disabled(0)
- *
- * @see SBasePlugin_setSBMLDocument
- * @see SBasePlugin_connectToParent
- *
- * @return integer value indicating success/failure of the
- * function.  @if clike The value is drawn from the
- * enumeration #OperationReturnValues_t. @endif@~ The possible values
- * returned by this function are:
- * @li @link OperationReturnValues_t#LIBSBML_OPERATION_SUCCESS LIBSBML_OPERATION_SUCCESS @endlink
- * @li @link OperationReturnValues_t#LIBSBML_INVALID_OBJECT LIBSBML_INVALID_OBJECT @endlink
- */
 LIBSBML_EXTERN
 int
 SBasePlugin_enablePackageInternal(SBasePlugin_t* plugin, 
@@ -1150,14 +922,6 @@ SBasePlugin_enablePackageInternal(SBasePlugin_t* plugin,
   return LIBSBML_OPERATION_SUCCESS;
 }
 
-/**
- * Returns the parent SBMLDocument of this plugin structure.
- *
- * @param plugin the SBasePlugin_t structure  
- *
- * @return the parent SBMLDocument object of this plugin object or NULL if 
- * no document is set, or the plugin structure is invalid. 
- */
 LIBSBML_EXTERN
 SBMLDocument_t*
 SBasePlugin_getSBMLDocument(SBasePlugin_t* plugin)
@@ -1166,14 +930,6 @@ SBasePlugin_getSBMLDocument(SBasePlugin_t* plugin)
   return plugin->getSBMLDocument();
 }
 
-/**
- * Returns the parent SBase structure to which this plugin structure is connected.
- *
- * @param plugin the SBasePlugin_t structure  
- *
- * @return the parent SBase structure to which this plugin structure is connected
- * or NULL if sbase structure is set, or the plugin structure is invalid. 
- */
 LIBSBML_EXTERN
 SBase_t*
 SBasePlugin_getParentSBMLObject(SBasePlugin_t* plugin)
@@ -1182,15 +938,6 @@ SBasePlugin_getParentSBMLObject(SBasePlugin_t* plugin)
   return plugin->getParentSBMLObject();
 }
 
-/**
- * Returns the SBML level of the package extension of 
- * this plugin structure.
- *
- * @param plugin the SBasePlugin_t structure  
- *
- * @return the SBML level of the package extension of
- * this plugin object or SBML_INT_MAX if the structure is invalid.
- */
 LIBSBML_EXTERN
 unsigned int
 SBasePlugin_getLevel(SBasePlugin_t* plugin)
@@ -1199,15 +946,6 @@ SBasePlugin_getLevel(SBasePlugin_t* plugin)
   return plugin->getLevel();
 }
 
-/**
- * Returns the SBML version of the package extension of 
- * this plugin structure.
- *
- * @param plugin the SBasePlugin_t structure  
- *
- * @return the SBML version of the package extension of
- * this plugin object or SBML_INT_MAX if the structure is invalid.
- */
 LIBSBML_EXTERN
 unsigned int
 SBasePlugin_getVersion(SBasePlugin_t* plugin)
@@ -1216,15 +954,6 @@ SBasePlugin_getVersion(SBasePlugin_t* plugin)
   return plugin->getVersion();
 }
 
-/**
- * Returns the package version of the package extension of 
- * this plugin structure.
- *
- * @param plugin the SBasePlugin_t structure  
- *
- * @return the package version of the package extension of
- * this plugin object or SBML_INT_MAX if the structure is invalid.
- */
 LIBSBML_EXTERN
 unsigned int
 SBasePlugin_getPackageVersion(SBasePlugin_t* plugin)
