@@ -8,8 +8,15 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright 2011 California Institute of Technology.
+ * Copyright (C) 2013-2014 jointly by the following organizations:
+ *     1. California Institute of Technology, Pasadena, CA, USA
+ *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
+ *     3. University of Heidelberg, Heidelberg, Germany
  * 
+ * Copyright 2011-2012 jointly by the following organizations:
+ *     1. California Institute of Technology, Pasadena, CA, USA
+ *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
+ *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published by
  * the Free Software Foundation.  A copy of the license agreement is provided
@@ -31,7 +38,7 @@
 #include <sbml/Model.h>
 
 #include <sbml/util/ElementFilter.h>
-#include <sbml/util/IdentifierTransformer.h>
+#include <sbml/util/PrefixTransformer.h>
 
 using namespace std;
 
@@ -44,6 +51,7 @@ CompModelPlugin::CompModelPlugin (const std::string &uri, const std::string &pre
   , mListOfPorts(compns)
   , mDivider("__")
   , mRemoved()
+  , mTransformer(NULL)
 {
   connectToChild();
 }
@@ -55,6 +63,7 @@ CompModelPlugin::CompModelPlugin(const CompModelPlugin& orig)
   , mListOfPorts(orig.mListOfPorts)
   , mDivider("__")
   , mRemoved() //If we're making a copy, the list of things we've removed is new.
+  , mTransformer(orig.mTransformer)
 {
   connectToChild();
 }
@@ -74,6 +83,7 @@ CompModelPlugin::operator=(const CompModelPlugin& orig)
     mListOfPorts = orig.mListOfPorts;
     mDivider = orig.mDivider;
     mRemoved.clear(); //If we're making a copy, the list of things we've removed is new.
+    mTransformer = orig.mTransformer;
     connectToChild();
   }
   return *this;
@@ -287,6 +297,7 @@ CompModelPlugin::createSubmodel ()
   COMP_CREATE_NS(compns, getSBMLNamespaces());
   Submodel* m = new Submodel(compns);
   mListOfSubmodels.appendAndOwn(m);
+  delete compns;
   return m;
 }
 
@@ -417,6 +428,7 @@ CompModelPlugin::createPort ()
   COMP_CREATE_NS(compns, getSBMLNamespaces());
   Port* m = new Port(compns);
   mListOfPorts.appendAndOwn(m);
+  delete compns;
   return m;
 }
 
@@ -560,6 +572,9 @@ Model* CompModelPlugin::flattenModel() const
     }
   }
 
+  // delete the list
+  delete allelements;
+
   for (unsigned int el=0; el<nonReplacedElements.size(); el++) 
   {
     SBase* element = nonReplacedElements[el];
@@ -572,6 +587,8 @@ Model* CompModelPlugin::flattenModel() const
     }
   }
 
+
+
   //Finally, unset the document again.
   flat->setSBMLDocument(NULL);
 
@@ -669,12 +686,21 @@ CompModelPlugin::instantiateSubmodels()
     Submodel* submodel = mListOfSubmodels.get(sub);
     // Instead of 'instantiate', since we might have already 
     // been instantiated ourselves from above.
-    const Model* submodinst = submodel->getInstantiation(); 
+    Model* submodinst = submodel->getInstantiation(); 
     
     if (submodinst == NULL ) {
       //'getInstantiation' already sets any errors that might have occurred.
       return LIBSBML_OPERATION_FAILED;
     }
+    
+    // if we have a transformer specified, then we need to propagate it, so it can
+    // be used
+    if (isSetTransformer())
+    {
+      CompModelPlugin* other = dynamic_cast<CompModelPlugin*>(submodinst->getPlugin("comp"));
+      if (other != NULL)
+        other->setTransformer(getTransformer());
+    }
   }
 
   // Next, recursively find all the targets of SBaseRef elements 
@@ -815,10 +841,12 @@ int CompModelPlugin::saveAllReferencedElements(set<SBase*> uniqueRefs, set<SBase
               }
               else 
               {
+                delete allElements;
                 return ret;
               }
             }
             else {
+              delete allElements;
               return ret;
             }
           }
@@ -841,6 +869,7 @@ int CompModelPlugin::saveAllReferencedElements(set<SBase*> uniqueRefs, set<SBase
                 error += " has a <replacedBy> child and is also pointed to by a <port>, <deletion>, <replacedElement>, or one or more <replacedBy> objects.";
                 doc->getErrorLog()->logPackageError("comp", CompNoMultipleReferences, getPackageVersion(), getLevel(), getVersion(), error);
               }
+              delete allElements;
               return LIBSBML_OPERATION_FAILED;
             }
             adddirect = replacedBys.insert(direct).second;
@@ -874,12 +903,15 @@ int CompModelPlugin::saveAllReferencedElements(set<SBase*> uniqueRefs, set<SBase
                 }
                 doc->getErrorLog()->logPackageError("comp", CompNoMultipleReferences, getPackageVersion(), getLevel(), getVersion(), error);
               }
+              delete allElements;
               return LIBSBML_OPERATION_FAILED;
             }
           }
     }
   }
 
+  delete allElements;
+
   //Now call saveAllReferencedElements for all instantiated submodels.
   for (unsigned long sm=0; sm<getNumSubmodels(); sm++) {
     Model* sub = getSubmodel(sm)->getInstantiation();
@@ -948,6 +980,7 @@ CompModelPlugin::renameAllIDsAndPrepend(const std::string& prefix)
     Model* inst = subm->getInstantiation();
     if (inst==NULL) {
       //'getInstantiation' will set its own error messages.
+      delete allElements;
       return LIBSBML_OPERATION_FAILED;
     }
     CompModelPlugin* instp = static_cast<CompModelPlugin*>(inst->getPlugin(getPrefix()));
@@ -957,20 +990,28 @@ CompModelPlugin::renameAllIDsAndPrepend(const std::string& prefix)
         string error = "Unable to rename elements in CompModelPlugin::renameAllIDsAndPrepend: no valid 'comp' plugin for the model instantiated from submodel " + subm->getId();
         doc->getErrorLog()->logPackageError("comp", CompModelFlatteningFailed, getPackageVersion(), getLevel(), getVersion(), error);
       }
+      delete allElements;
       return LIBSBML_OPERATION_FAILED;
     }
     int ret = instp->renameAllIDsAndPrepend(prefix + submodids[sm]);
     if (ret != LIBSBML_OPERATION_SUCCESS) {
       //'renameAllIds..' will set its own error messages.
+      delete allElements;
       return ret;
     }
   }
 
   //Finally, actually rename the elements in *this* model with the prefix.
-  if (prefix.empty()) return LIBSBML_OPERATION_SUCCESS; //Nothing to add
+  if (prefix.empty()) 
+  {
+    delete allElements;
+    return LIBSBML_OPERATION_SUCCESS; //Nothing to add
+  }
 
   //Rename the SIds, UnitSIds, and MetaIDs, and references to them.
   renameIDs(allElements, prefix);
+  delete allElements;
+
   return LIBSBML_OPERATION_SUCCESS;
 }
 
@@ -1035,43 +1076,6 @@ void CompModelPlugin::findUniqueSubmodPrefixes(vector<string>& submodids, List*
 }
 /** @endcond */
 
-/** 
- * Simple IdentifierTransformer, that prefixes all given 
- * elements with the prefix given to the constructor. 
- * 
- * this will prefix metaids, unitsids and sids. 
- */ 
-class PrefixTransformer : public IdentifierTransformer
-{
-  string mPrefix;
-public: 
-  PrefixTransformer (const string& prefix) 
-  : mPrefix(prefix) {}
-  
-  virtual int transform(SBase* element)
-  {
-    // if there is nothing to do return ... 
-    if (element == NULL) 
-    return LIBSBML_OPERATION_SUCCESS;
-  
-    // prefix meta id if we have one ... 
-    if (element->isSetMetaId())
-    {
-      if (element->setMetaId(mPrefix + element->getMetaId()) != LIBSBML_OPERATION_SUCCESS)
-        return LIBSBML_OPERATION_FAILED;
-    }
-
-    // prefix other ids (unitsid, or sid) ...
-    // skip local parameters
-    if (element->isSetId() && element->getTypeCode() != SBML_LOCAL_PARAMETER)
-    {
-      if (element->setId(mPrefix + element->getId()) != LIBSBML_OPERATION_SUCCESS)
-        return LIBSBML_OPERATION_FAILED;
-    }
-    return LIBSBML_OPERATION_SUCCESS;
-  }
-};
-
 /** @cond doxygenLibsbmlInternal */
 void CompModelPlugin::renameIDs(List* allElements, const string& prefix)
 {
@@ -1079,14 +1083,25 @@ void CompModelPlugin::renameIDs(List* allElements, const string& prefix)
   vector<pair<string, string> > renamedSIds;
   vector<pair<string, string> > renamedUnitSIds;
   vector<pair<string, string> > renamedMetaIds;
-  //PrefixTransformer trans(prefix);
+  
+  // if a custom prefix transformer was specified, then set the 
+  // current prefix
+  if (isSetTransformer())
+    mTransformer->setPrefix(prefix);
+  
   for (unsigned long el=0; el < allElements->getSize(); ++el) 
   {
     SBase* element = static_cast<SBase*>(allElements->get(el));
     string id = element->getId();
     string metaid = element->getMetaId();
-    //element->transformIdentifiers(&trans);
-    element->prependStringToAllIdentifiers(prefix);
+    
+    // if a custom prefix transformer was specified, use it, other wise
+    // default to the sbase method. 
+    if (isSetTransformer())
+      element->transformIdentifiers(mTransformer);
+    else 
+      element->prependStringToAllIdentifiers(prefix);
+
     if (element->getTypeCode() == SBML_LOCAL_PARAMETER) {
       element->setId(id); //Change it back.  This would perhaps be better served by overriding 'prependStringToAllIdentifiers' but hey.
     }
@@ -1235,6 +1250,7 @@ int CompModelPlugin::collectRenameAndConvertReplacements(set<SBase*>* removed, s
       rbs.push_back(reference);
     }
   }
+  delete allElements;
 
   //ReplacedElement replacements
   for (size_t re=0; re<res.size(); re++) {
@@ -1299,6 +1315,7 @@ int CompModelPlugin::removeCollectedElements(set<SBase*>* removed, set<SBase*>*
         SBase* element = static_cast<SBase*>(children->get(el));
         removed->insert(element);
       }
+      delete children;
       CompBase::removeFromParentAndPorts(removeme, removed);
     }
     toremove->erase(removeme);
@@ -1331,6 +1348,33 @@ CompModelPlugin::accept(SBMLVisitor& v) const
 
 /** @endcond */
 
+void 
+CompModelPlugin::setTransformer(PrefixTransformer* transformer)
+{
+  mTransformer = transformer;
+}
+
+PrefixTransformer* 
+CompModelPlugin::getTransformer() const
+{
+  return mTransformer;
+}
+
+bool 
+CompModelPlugin::isSetTransformer() const
+{
+  return mTransformer != NULL;
+}
+
+
+void 
+CompModelPlugin::unsetTransformer()
+{
+  mTransformer = NULL;
+}
+
+
+
 /** @cond doxygenLibsbmlInternal */
 std::set<SBase*>* 
 CompModelPlugin::getRemovedSet() 
@@ -1340,7 +1384,7 @@ CompModelPlugin::getRemovedSet()
 /** @endcond */
 
 #endif /* __cplusplus */
-/** @cond doxygenCOnly */
+/** @cond doxygenIgnored */
 LIBSBML_EXTERN
 Submodel_t *
 CompModelPlugin_createSubmodel(CompModelPlugin_t * modelPlug)
