@@ -8,9 +8,14 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
+ * Copyright (C) 2013-2014 jointly by the following organizations:
+ *     1. California Institute of Technology, Pasadena, CA, USA
+ *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
+ *     3. University of Heidelberg, Heidelberg, Germany
+ * 
  * Copyright (C) 2009-2011 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
- *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
+ *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *  
  * Copyright (C) 2006-2008 by the California Institute of Technology,
  *     Pasadena, CA, USA 
@@ -29,6 +34,8 @@
 
 #include <sbml/packages/qual/extension/QualModelPlugin.h>
 #include <sbml/packages/qual/validator/QualSBMLError.h>
+#include <sbml/util/ElementFilter.h>
+#include <sbml/Model.h>
 
 #include <iostream>
 using namespace std;
@@ -425,6 +432,8 @@ QualModelPlugin::createQualitativeSpecies ()
   {  
     QUAL_CREATE_NS(qualns, getSBMLNamespaces());
     g = new QualitativeSpecies(qualns);
+    mQualitativeSpecies.appendAndOwn(g);
+    delete qualns;
   }
   catch(...)
   {
@@ -437,8 +446,6 @@ QualModelPlugin::createQualitativeSpecies ()
      */
   }
 
-  if (g) mQualitativeSpecies.appendAndOwn(g);
-
   return g;
 }
 /** @endcond */
@@ -610,6 +617,8 @@ QualModelPlugin::createTransition ()
   {  
     QUAL_CREATE_NS(qualns, getSBMLNamespaces());
     g = new Transition(qualns);
+    mTransitions.appendAndOwn(g);
+    delete qualns;
   }
   catch(...)
   {
@@ -622,8 +631,6 @@ QualModelPlugin::createTransition ()
      */
   }
 
-  if (g) mTransitions.appendAndOwn(g);
-
   return g;
 }
 /** @endcond */
